@@ -15,27 +15,15 @@
 //! # Usage
 //!
 //! This crate is a dependency of the [milter][crate] crate, which re-exports
-//! all macros under its namespace. Thus you should not need to use this crate
-//! directly.
-//!
-//! Note that with Rust 2018 there are two ways of importing procedural macros.
-//! Nowadays, macros can be imported like other symbols:
+//! all macros under its namespace. It is recommended to `use` macros from the
+//! `milter` namespace and not rely on this crate directly:
 //!
 //! ```
 //! use milter::{on_connect, on_close, Milter, Status};
 //! ```
 //!
-//! That does require you to list all macros in the `use` statement, though.
-//! The older syntax with `extern crate` lets you import all macros without
-//! listing them explicitly:
-//!
-//! ```
-//! #[macro_use] extern crate milter;
-//!
-//! use milter::{Milter, Status};
-//! ```
-//!
-//! Both notations are acceptable.
+//! The remaining sections describe some additional features of the attribute
+//! macros.
 //!
 //! # Raw string inputs
 //!
@@ -44,7 +32,7 @@
 //! the C string type `&CStr` for `&str` in your handler function signature in
 //! order to receive the raw bytes instead.
 //!
-//! Contrast the following example with the one shown at [`on_header`].
+//! Contrast the following example with the one shown at [`macro@on_header`].
 //!
 //! ```
 //! # #[macro_use] extern crate milter_callback;
@@ -68,7 +56,7 @@
 //! methods return `milter::Result`s these can then be unwrapped with the `?`
 //! operator.
 //!
-//! Compare the following example with the one shown at [`on_eom`]. This code
+//! Compare the following example with the one shown at [`macro@on_eom`]. This code
 //! fragment also demonstrates the use of the `?` operator enabled by choosing
 //! this return type.
 //!
@@ -97,28 +85,24 @@
 //! Panicking, on the other hand, leads to immediate [shutdown] of the milter.
 //! All stages switch to returning a failure response and no longer execute the
 //! handler functions (however, currently executing callback handlers are
-//! allowed to finish). The milter library worker processes are terminated and
-//! the currently blocked invocation of `Milter::run` returns. Cleanup logic in
-//! the `close` or other stages is not executed.
+//! allowed to finish). The libmilter worker processes are terminated and the
+//! currently blocked invocation of `Milter::run` returns. Cleanup logic in the
+//! `close` or other stages is not executed.
 //!
 //! The principle behind the panicking behaviour is, as elsewhere, exit as
-//! quickly as possible, within the constraints posed by the milter library and
-//! the FFI interface.
+//! quickly as possible, within the constraints posed by libmilter and the FFI
+//! interface.
 //!
 //! The above failure modes are provided as a convenience. Use explicit error
 //! handling if they don’t satisfy your requirements.
 //!
-//! [milter]: https://docs.rs/milter/0.2.3/milter
-//! [`Milter`]: https://docs.rs/milter/0.2.3/milter/struct.Milter.html
+//! [milter]: https://docs.rs/milter/0.2.4/milter
+//! [`Milter`]: https://docs.rs/milter/0.2.4/milter/struct.Milter.html
 //! [attribute macro]: #attributes
 //! [crate]: https://crates.io/crates/milter
-//! [`on_header`]: attr.on_header.html
-//! [`milter::Result`]: https://docs.rs/milter/0.2.3/milter/type.Result.html
-//! [`on_eom`]: attr.on_eom.html
-//! [`Status::Tempfail`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html#variant.Tempfail
-//! [shutdown]: https://docs.rs/milter/0.2.3/milter/fn.shutdown.html
-
-#![doc(html_root_url = "https://docs.rs/milter-callback/0.2.3")]
+//! [`milter::Result`]: https://docs.rs/milter/0.2.4/milter/type.Result.html
+//! [`Status::Tempfail`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html#variant.Tempfail
+//! [shutdown]: https://docs.rs/milter/0.2.4/milter/fn.shutdown.html
 
 mod generator;
 mod tree_preds;
@@ -185,13 +169,13 @@ macro_rules! c_type {
 /// }
 /// ```
 ///
-/// [`NegotiateCallback`]: https://docs.rs/milter/0.2.3/milter/type.NegotiateCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Actions`]: https://docs.rs/milter/0.2.3/milter/struct.Actions.html
-/// [`ProtocolOpts`]: https://docs.rs/milter/0.2.3/milter/struct.ProtocolOpts.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
-/// [`Status::AllOpts`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html#variant.AllOpts
-/// [`Status::Continue`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html#variant.Continue
+/// [`NegotiateCallback`]: https://docs.rs/milter/0.2.4/milter/type.NegotiateCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Actions`]: https://docs.rs/milter/0.2.4/milter/struct.Actions.html
+/// [`ProtocolOpts`]: https://docs.rs/milter/0.2.4/milter/struct.ProtocolOpts.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
+/// [`Status::AllOpts`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html#variant.AllOpts
+/// [`Status::Continue`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html#variant.Continue
 #[proc_macro_attribute]
 pub fn on_negotiate(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -259,12 +243,10 @@ pub fn on_negotiate(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`ConnectCallback`]: https://docs.rs/milter/0.2.3/milter/type.ConnectCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [str]: https://doc.rust-lang.org/std/primitive.str.html
-/// [Option]: https://doc.rust-lang.org/std/option/enum.Option.html
-/// [SocketAddr]: https://doc.rust-lang.org/std/net/enum.SocketAddr.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`ConnectCallback`]: https://docs.rs/milter/0.2.4/milter/type.ConnectCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [SocketAddr]: std::net::SocketAddr
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_connect(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -311,10 +293,9 @@ pub fn on_connect(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`HeloCallback`]: https://docs.rs/milter/0.2.3/milter/type.HeloCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [str]: https://doc.rust-lang.org/std/primitive.str.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`HeloCallback`]: https://docs.rs/milter/0.2.4/milter/type.HeloCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_helo(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -360,11 +341,9 @@ pub fn on_helo(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`MailCallback`]: https://docs.rs/milter/0.2.3/milter/type.MailCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [Vec]: https://doc.rust-lang.org/std/vec/struct.Vec.html
-/// [str]: https://doc.rust-lang.org/std/primitive.str.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`MailCallback`]: https://docs.rs/milter/0.2.4/milter/type.MailCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_mail(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -410,11 +389,9 @@ pub fn on_mail(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`RcptCallback`]: https://docs.rs/milter/0.2.3/milter/type.RcptCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [Vec]: https://doc.rust-lang.org/std/vec/struct.Vec.html
-/// [str]: https://doc.rust-lang.org/std/primitive.str.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`RcptCallback`]: https://docs.rs/milter/0.2.4/milter/type.RcptCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_rcpt(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -458,9 +435,9 @@ pub fn on_rcpt(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`DataCallback`]: https://docs.rs/milter/0.2.3/milter/type.DataCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`DataCallback`]: https://docs.rs/milter/0.2.4/milter/type.DataCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_data(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -506,10 +483,9 @@ pub fn on_data(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`HeaderCallback`]: https://docs.rs/milter/0.2.3/milter/type.HeaderCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [str]: https://doc.rust-lang.org/std/primitive.str.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`HeaderCallback`]: https://docs.rs/milter/0.2.4/milter/type.HeaderCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_header(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -553,9 +529,9 @@ pub fn on_header(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`EohCallback`]: https://docs.rs/milter/0.2.3/milter/type.EohCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`EohCallback`]: https://docs.rs/milter/0.2.4/milter/type.EohCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_eoh(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -601,9 +577,9 @@ pub fn on_eoh(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`BodyCallback`]: https://docs.rs/milter/0.2.3/milter/type.BodyCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`BodyCallback`]: https://docs.rs/milter/0.2.4/milter/type.BodyCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_body(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -650,10 +626,10 @@ pub fn on_body(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`EomCallback`]: https://docs.rs/milter/0.2.3/milter/type.EomCallback.html
-/// [actions]: https://docs.rs/milter/0.2.3/milter/struct.Actions.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`EomCallback`]: https://docs.rs/milter/0.2.4/milter/type.EomCallback.html
+/// [actions]: https://docs.rs/milter/0.2.4/milter/struct.Actions.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_eom(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -696,9 +672,9 @@ pub fn on_eom(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`AbortCallback`]: https://docs.rs/milter/0.2.3/milter/type.AbortCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`AbortCallback`]: https://docs.rs/milter/0.2.4/milter/type.AbortCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_abort(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -744,9 +720,9 @@ pub fn on_abort(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`CloseCallback`]: https://docs.rs/milter/0.2.3/milter/type.CloseCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`CloseCallback`]: https://docs.rs/milter/0.2.4/milter/type.CloseCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_close(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
@@ -789,10 +765,9 @@ pub fn on_close(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
-/// [`UnknownCallback`]: https://docs.rs/milter/0.2.3/milter/type.UnknownCallback.html
-/// [Context]: https://docs.rs/milter/0.2.3/milter/struct.Context.html
-/// [str]: https://doc.rust-lang.org/std/primitive.str.html
-/// [`Status`]: https://docs.rs/milter/0.2.3/milter/enum.Status.html
+/// [`UnknownCallback`]: https://docs.rs/milter/0.2.4/milter/type.UnknownCallback.html
+/// [Context]: https://docs.rs/milter/0.2.4/milter/struct.Context.html
+/// [`Status`]: https://docs.rs/milter/0.2.4/milter/enum.Status.html
 #[proc_macro_attribute]
 pub fn on_unknown(attr: TokenStream, item: TokenStream) -> TokenStream {
     let name = parse_macro_input!(attr as Ident);
